
// Draw a cylinder centered on the y-axis, going down from y=0 to y=height. 
// The radius at the top can be different from the radius at the bottom, 
// and the number of sides drawn is variable. 

import oscP5.*; //OpenSoundControl: http://www.sojamo.de/libraries/oscP5/
import netP5.*;
import processing.serial.*; //Accelerometer data from Arduino
import fullscreen.*;        // For updates: http://www.superduper.org/processing/fullscreen_api/
import processing.opengl.*;  //OPENGL is in an experimental mode when entering fullscreen
import quicktime.std.movies.MovieDrawingComplete; // This enable fullscreen and exit fullscreen
import quicktime.std.movies.MovieController;

//OpenSpundControl
OscP5 oscP5;
NetAddress myRemoteLocation;
float[] pa;   //PhidgetsAccelerometer data from Max/MSP.
int[] cc;   //cc = change color.Accelerometer data from 
              //Arduino via Processing to Max/MSP and back to Processing.

//Accelerometer data from Arduino
Serial myPort;
 char inByte;
 int xVal, yVal, zVal = 0;
 int[] serialInArray = new int[3];
 int serialCount = 0;
 boolean firstContact = true;
 String theValues = null; //string holds

//This enable fullscreen.
//Keyboard shortcuts: To enter/leave fullscreen mode: Comand+F and esc to exit screen.
FullScreen fs; 

void setup() {

  // set size to 1440X900, MacBook pro 15"
  size(1440, 900, OPENGL);

  // 5 fps
  frameRate(24);

  //Max/MSP is listening to port 11000
  oscP5 = new OscP5(this,11000); 
  //Processing is liatening to port 11001.
  myRemoteLocation = new NetAddress("127.0.0.1",11001);
  
  
  //Receive an int when Max/MSP is operative 
  oscP5.plug(this, "connected", "/max/connected");
  //Receive OSC data from Max/MSP: PhidgetsAccelerometer.  
  oscP5.plug(this, "pa","/max/paData"); 
  pa = new float[3];
  //Data from Max/MSP 
  oscP5.plug(this, "cc", "/max/colorChange");
  cc = new int[3];

  // Create the fullscreen object
  // enter fullscreen mode
  fs = new FullScreen(this); 
  fs.enter(); 


  //This will hide the mouse cursor
  try {
    Robot robot = new Robot();
    robot.mouseMove(int(width),int(height));
  } 
  catch (AWTException e) {
  }
  //setup serial port
  myPort = new Serial(this, Serial.list()[0], 9600);
   	myPort.clear();


}

//This method prints out connection status when Max/MSP is operative.
void connected(int theFlag) {
  if(theFlag==1) {
    println("Max/MSP connected");
  }
  else {
    println("Max/MSP DISCONNECTED");
  }
}

//This method receive 3 PhidgetsAccelerometer float values from Max/MSP to navigate the object. 
void pa(float x_val, float y_val, float z_val) {
  pa[0] = x_val;
  pa[1] = y_val;
  pa[2] = z_val;
}

//This method receive 3 Accelerometer int values from Max/MSP to change color.
void cc(int x_color, int y_color, int z_color) {
  cc[0] = x_color;
  cc[1] = y_color;
  cc[2] = z_color;
}

void draw() {
  background(255, 255, 255);
  fill(cc[0],cc[1] ,cc[2]);//Use accelerometer data to change color
  lights(); 

  translate(width/2, height/2); 
  rotateX(pa[2]*PI/350);
  rotateY(pa[1]*PI/400);
  rotateZ(pa[0]*PI/350);
   
  //rect(pa[0], pa[1], 250, 250);
  noStroke();
  translate(0, -40, 0); 

  drawCylinder(pa[1], pa[0], pa[2], 200);  // Draw a cylinder and defrom the object! 

}

void drawCylinder(float topRadius, float bottomRadius, float tall, int sides) { // The object is born
  float angle = 0; 
  float angleIncrement = TWO_PI / sides; 
  beginShape(QUAD_STRIP); 
  for (int i = 0; i < sides + 1; ++i) { 
    vertex(topRadius*cos(angle), 0, topRadius*sin(angle)); 
    vertex(bottomRadius*cos(angle), tall, bottomRadius*sin(angle)); 
    angle += angleIncrement; 
  } 
  endShape(); 
  // If it is not a cone, draw the circular top cap 
  if (topRadius != 0) { 
    angle = 0; 
    beginShape(TRIANGLE_FAN); 
    // Center point 
    vertex(0, 0, 0); 
    for (int i = 0; i < sides + 1; i++) { 
      vertex(topRadius * cos(angle), 0, topRadius * sin(angle)); 
      angle += angleIncrement; 
    }  
    endShape(); 
  } 
  // If it is not a cone, draw the circular bottom cap 
  if (bottomRadius != 0) { 
    angle = 0; 
    beginShape(TRIANGLE_FAN); 
    // Center point
    vertex(0, tall, 0); 
    for (int i = 0; i < sides+1; i++) { 
      vertex(bottomRadius * cos(angle), tall, bottomRadius * sin(angle)); 
      angle += angleIncrement; 
    }  
    endShape(); 
  } 
}

void serialEvent(Serial port) {
 	// read a byte from the serial port:
 	int inByte = port.read();
 	// if this is the first byte received, 
 	// take note of that fact. Otherwise, add it to the array:
 	if (firstContact == false) {
 		if (inByte == 'A') { 
 			port.clear();          // clear the serial port buffer
 			firstContact = true;
 			port.write('A');
 		} 
 	} else {
 		// Add the latest byte from the serial port to array:
 		serialInArray[serialCount] = inByte;
 		serialCount++;
 
 		// If we have 3 bytes:
 		if (serialCount > 2 ) {
 			xVal = serialInArray[0];
 			yVal = serialInArray[1];
 			zVal = serialInArray[2];
 
 			//Sends accelerometer data from Arduino via Processing to Max/MSP
 			OscMessage myMessage = new OscMessage("/Arduino");   
 			myMessage.add(xVal);  
 			myMessage.add(yVal);  
 			myMessage.add(zVal);  
 			oscP5.send(myMessage, myRemoteLocation);
 
 			// print the values (for debugging purposes only):
 			//println(xVal + "\t" + yVal + "\t" + zVal);
 
 			// Send a capital A to request new sensor readings:
 			port.write('A');
 			// Reset serialCount:
 			serialCount = 0;
 		}
 	}
 
 }
